package edu.kit.ipd.descartes.adaptation.evaluation.performance.weightingfunction;

import org.apache.log4j.Logger;

import edu.kit.ipd.descartes.mm.adaptation.WeightedMetric;
import edu.kit.ipd.descartes.mm.adaptation.WeightedTactic;
import edu.kit.ipd.descartes.mm.adaptation.WeightingFunction;
import edu.kit.ipd.descartes.perfdatarepo.Impact;
import edu.kit.ipd.descartes.perfdatarepo.MetricValue;
import edu.kit.ipd.descartes.perfdatarepo.Result;

public class BlueYonderResourceEfficiencyWeightingFunction implements IWeightingFunction {

    private static final String UTIL_IDENTIFIER = "Util";
    private static final double THRESHOLD = 0.05;
    private WeightingFunction function = null;
    private static Logger logger = Logger.getLogger(BlueYonderResourceEfficiencyWeightingFunction.class);

//    public BlueYonderResourceEfficiencyWeightingFunction() {
//        AbstractEcoreModelLoader loader = PerformanceDataRepositoryModelLoader.getInstance();
//        String pdrLocation = loader.getCurrentModelUri().toFileString();
//        logger.debug("Initializing query engine adaptor for performance data repository " + pdrLocation);
//        qea = new QueryEngineAdapter(pdrLocation);
//    }

    @Override
    public void setWeightingFunction(WeightingFunction function) {
        this.function = function;
    }

    @Override
    /*
     * (non-Javadoc)
     * 
     * @see
     * edu.kit.ipd.descartes.adaptation.evaluation.performance.weightingfunction.IWeightingFunction#calculateWeight
     * (edu.kit.ipd.descartes.mm.adaptation.WeightedTactic)
     */
    public double calculateWeight(WeightedTactic weightedTactic) {
        double newWeight = 0.0;
        Impact impact = weightedTactic.getLastImpact();
        Result before = impact.getBefore();
        Result after = impact.getAfter();

        for (WeightedMetric wm : function.getWeightedMetrics()) {
            double afterValue = WeightedSumWeightingFunctionHelper.getValueForMetricType(wm.getMetricType(), after);
            double beforeValue = WeightedSumWeightingFunctionHelper.getValueForMetricType(wm.getMetricType(), before);
            double delta = afterValue - beforeValue;
            newWeight += delta * wm.getWeight();
        }
        
        int activeResourceDelta = calcActiveResourceDelta(impact);
        logger.debug("Delta of active resources: " + activeResourceDelta);
        // TODO: Implement something more sophisticated
        newWeight = activeResourceDelta;
        
        return newWeight;
    }

    private int calcActiveResourceDelta(Impact impact) {
        Result before = impact.getBefore();
        Result after = impact.getAfter();
        int activeResourcesBefore = getActiveResources(before);
        int activeResourcesAfter = getActiveResources(after);
        return activeResourcesAfter - activeResourcesBefore;
    }

    private int getActiveResources(Result result) {
        int counter = 0;
        for (MetricValue v : result.getMetricValues()) {
            // TODO parse with regexp or something more clever
            if (v.getMetricType().getName().contains(UTIL_IDENTIFIER) && v.getValue() > THRESHOLD)
                counter++;
        }
        return counter;
    }

}
