package edu.kit.ipd.descartes.adaptation.model.adaptation.dmm.util;

import java.util.List;

import org.eclipse.emf.common.util.EList;

import edu.kit.ipd.descartes.mm.adaptation.Event;
import edu.kit.ipd.descartes.mm.adaptation.RelationalOperator;
import edu.kit.ipd.descartes.mm.adaptation.Specification;
import edu.kit.ipd.descartes.perfdatarepo.MetricType;
import edu.kit.ipd.descartes.perfdatarepo.MetricValue;
import edu.kit.ipd.descartes.perfdatarepo.Result;

public class ObjectivesHelper {

    public static boolean compare(double left, double right, RelationalOperator operator) {
        switch (operator) {
        case EQUAL:
            return left == right;
        case NOT_EQUAL:
            return left != right;
        case LESS:
            return left < right;
        case LESS_EQUAL:
            return left <= right;
        case GREATER:
            return left > right;
        case GREATER_EQUAL:
            return left >= right;
        default:
            return false;
        }
    }

    public static boolean isObjectiveFulfilled(Event event, Result result) {
        List<Specification> specs = event.getTriggers().getObjective().getSpecifications();
        for (Specification specification : specs) {
            MetricType m = specification.getMetricType();
//          TODO: Connect QueryEngine:  Double actualValue = QueryEngineAdapter.getValueForMetricType(m);
            Double actualValue = getValueForMetricType(m, result);
            Double threshold = specification.getValue();
            if (!ObjectivesHelper.compare(actualValue, threshold, specification.getRelationalOperator()))
                return false;

        }
        return true;
    }
    
    private static double getValueForMetricType(MetricType metricType, Result result) {
        EList<MetricValue> metricValues = result.getMetricValues();

        for (MetricValue metricValue : metricValues) {
            if (sameMetricType(metricValue, metricType))
                return metricValue.getValue();
        }
        return Double.NaN;
    }

    private static boolean sameMetricType(MetricValue metricValue, MetricType metricType) {
//      return metricValue.getMetricType().equals(metricType);
      return (metricValue.getMetricType().getName().equals(metricType.getName())); 
//              && metricValue.getMetricType().getId().equals(metricType.getId()));
  }
    
}
